#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Classify student assignment submissions.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework extended
:version: 1.0.0
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""


import csv
import glob
import json
import os
import sys

from nltk import word_tokenize

from mqe_common import categorise_tokens, load_taxonomy

from mqe_regexs import GET_MA, GET_MODULE_CODE, GET_PID, GET_PRESENTATION_CODE


def classify_assignment_texts(source_folder='../assignments/',
                              taxonomies_folder='../taxonomies',
                              taxon_set='bloom'):
    """Apply classification to assignments."""
    results = []
    taxonomy = load_taxonomy(taxon_set, taxonomies_folder)
    txt_filepaths = glob.iglob(os.path.join(source_folder, '**/*.txt'),
                               recursive=True)
    for txt_filepath in txt_filepaths:
        print('Classifying {}'.format(txt_filepath), flush=True)
        json_filepath = txt_filepath[:-4] + '.json'
        source_text = get_text(txt_filepath)
        tokenised_text = word_tokenize(source_text)
        categories = categorise_tokens(tokenised_text, taxonomy)
        write_categories_json(json_filepath, categories)
        results.append(accumulate_results(json_filepath, categories))
        print('Classified {}'.format(json_filepath), flush=True)
    write_results_json(source_folder, results)
    write_results_tsv(source_folder, results)


def get_text(text_filepath):
    """Return text content."""
    with open(text_filepath, 'r', errors='replace') as f_in:
        return f_in.read()


def write_categories_json(json_filepath, categories):
    """Write categories to specified file."""
    with open(json_filepath, 'w', encoding='utf-8', newline='\n') as f_out:
        json.dump(categories, f_out)


def accumulate_results(json_filepath, categories):
    """Return extended summary line of results."""
    categories['module'] = GET_MODULE_CODE(json_filepath).group()
    categories['presentation'] = GET_PRESENTATION_CODE(json_filepath).group()
    categories['pid'] = GET_PID(json_filepath).group()
    categories['assignment'] = GET_MA(json_filepath).group()
    _, tail = os.path.split(json_filepath)
    categories['filename'] = tail[:-5]
    return categories


def write_results_json(source_folder, results):
    """Write categories as json to specified file."""
    results_filepath = os.path.join(source_folder, 'results.json')
    with open(results_filepath, 'w', encoding='utf-8', newline='\n') as f_json:
        json.dump(results, f_json)
    print('Results written to {}'.format(results_filepath))


def write_results_tsv(source_folder, results):
    """Write categories as tsv to specified file."""
    results_filepath = os.path.join(source_folder, 'results.tsv')
    with open(results_filepath, 'w', encoding='utf-8', newline='\n') as f_tsv:
        fieldnames = results[0].keys()
        writer = csv.DictWriter(f_tsv,
                                dialect='excel-tab',
                                fieldnames=fieldnames)
        writer.writeheader()
        writer.writerows(results)
    print('Results written to {}'.format(results_filepath))


if __name__ == '__main__':
    # sys.exit(classify_assignment_texts(source_folder='../outputTEST/'))
    sys.exit(classify_assignment_texts())
