#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Classify forums converted from XML to stemmed bag of words with metadata.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:version: 1.0.0
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""


import html
import os
import pickle
import sys
from datetime import date

from nltk import word_tokenize

from mqe_common import (categorise_tokens, get_forum_data,
                        is_current, load_taxonomy, set_folder)

from mqe_regexs import STRIP_TAGS


def classify_posts(source_folder,
                   taxon_set='bloom',
                   taxonomies_folder='../taxonomies',
                   results_folder='../forum-posts-classified/'):
    """Apply classification to forum posts XML files in source folder.."""
    print('Classifying forum posts in {}'.format(source_folder),
          flush=True)
    set_folder(results_folder)
    taxonomy = load_taxonomy(taxon_set, taxonomies_folder)
    for forum_name, posts in get_forum_data(source_folder):
        print('Processing {}'.format(forum_name), flush=True)
        output_file = os.path.join(results_folder, forum_name + '.pkl')
        with open(output_file, 'wb') as fout:
            pickle.dump(process_posts(posts, forum_name, taxon_set, taxonomy),
                        fout, pickle.HIGHEST_PROTOCOL)


def process_posts(posts, forum_name, taxon_set, taxonomy):
    """Return stemmed message and metadata for forum posts."""
    categorised_posts = []
    current_posts = filter(is_current, posts)
    for post in current_posts:
        categorised_post = {}
        categorised_post['postid'] = post.get('id')
        categorised_post['userid'] = post.find('userid').text
        created = post.find('created')
        post_date = date.fromtimestamp(int(created.text))
        categorised_post['created'] = post_date
        categorised_post['forum'] = forum_name
        message_text = post.find('message').text
        categorised_post['surface'] = message_text
        tokenised_message = tokenise_message(message_text)
        categorised_post['tokens'] = tokenised_message
        categorised_post[taxon_set] = categorise_tokens(tokenised_message,
                                                        taxonomy)
        categorised_posts.append(categorised_post)
    return categorised_posts


def tokenise_message(message_text):
    """Return unique stemmed tokens of message text."""
    message_unescaped = html.unescape(message_text)
    message_tagless = STRIP_TAGS(' ', message_unescaped)
    message_tokens = word_tokenize(message_tagless)
    return message_tokens


if __name__ == '__main__':
    sys.exit(classify_posts('../forums/'))
    # sys.exit(classify_posts('../forums/',
                            # taxon_set='solo',
                            # results_folder='../posts-classified-solo/'))
