#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Classify teaching material.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework extended
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""


from collections import Counter
import glob
import os.path
import sys

import nltk

from mqe_common import load_taxonomy
import extract_text_from_docx

STEMMER = nltk.LancasterStemmer()


def classify_teaching_text(source_folder='../Learning Design',
                           taxonomies_folder='../taxonomies',
                           taxon_set='lda'):
    """Apply classification to source material."""
    taxonomy = load_taxonomy(taxon_set, taxonomies_folder)
    docx_files = glob.iglob(os.path.join(source_folder, '**/*.docx'),
                            recursive=True)
    for docx_file in docx_files:
        print('Classifying {}'.format(docx_file), flush=True)
        docx_root, _ = os.path.splitext(docx_file)

        source_filepath = docx_root + '_plain.txt'
        source_text = extract_text_from_docx.get_text(docx_file)
        write_text(source_filepath, source_text)

        tokenised_filepath = docx_root + '_tokenised.txt'
        tokenised_text = nltk.word_tokenize(source_text)
        write_text(tokenised_filepath, str(tokenised_text))

        categorised_filepath = docx_root + '_categorised.txt'
        categorised_text = categorise_text(tokenised_text, taxonomy)
        write_text(categorised_filepath, str(categorised_text))


def write_text(text_filepath, text_contents):
    """Write text contents to specified file."""
    with open(text_filepath, 'w', encoding='utf-8', newline='\n') as f_out:
        f_out.write(text_contents)


def categorise_text(tokenised_text, taxonomy):
    """Return count of matched categorisable terms in source material."""
    stemmed_tokens = [STEMMER.stem(token) for token in tokenised_text]
    stemmed_counts = Counter(stemmed_tokens)
    category_counts = {key: 0 for key in taxonomy.keys()}
    for key, keywords in taxonomy.items():
        for keyword in keywords:
            if keyword in stemmed_counts:
                category_counts[key] += stemmed_counts[keyword]
    return category_counts


if __name__ == '__main__':
    sys.exit(classify_teaching_text())
    # sys.exit(classify_teaching_text(
        # '../Learning Design/M811_17k_content/Assessment Guide'))
