#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Extract text contents from docx files.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework extended
:version: 1.0.0
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""


import glob
import os
import sys
import zipfile
from xml.etree.ElementTree import XML


WORD_NAMESPACE = '{http://schemas.openxmlformats.org/wordprocessingml/2006/main}'
PARA = WORD_NAMESPACE + 'p'
TEXT = WORD_NAMESPACE + 't'


def get_text(path):
    """Return the text content of a docx file."""
    try:
        with zipfile.ZipFile(path) as f_zip:
            xml_content = f_zip.read('word/document.xml')
            tree = XML(xml_content)

        paragraphs = []
        for paragraph in tree.iter(PARA):
            texts = [node.text
                     for node in paragraph.iter(TEXT)
                     if node.text]
            if texts:
                paragraphs.append(''.join(texts))

        return '\n\n'.join(paragraphs)

    except zipfile.BadZipFile:
        print('Not a valid file, processing bypassed', flush=True)
        return 'Not a valid file, processing bypassed'


def extract_text_from_docx(source_folder='../assignments/',
                           pattern='**/*.docx'):
    """Extract the text content of every .docx file in source folder."""
    filenames = glob.iglob(os.path.join(source_folder, pattern), recursive=True)
    for filename in filenames:
        print('Reading {}'.format(filename))
        if 'MACOSX' in filename:
            print('*** file not processed')
            continue
        text_filename = filename[:-5] + '.txt'
        print('Writing {}'.format(text_filename))
        with open(text_filename, 'w',
                  encoding='utf-8', newline='\n') as fout:
            fout.write(get_text(filename))


if __name__ == '__main__':
    sys.exit(extract_text_from_docx())
