#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Search VLE forum posts to exclude those that use external references.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:version: 1.0.0
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

import os
import sys
from datetime import date

from mqe_common import get_forum_data, has_link, is_current, set_folder

from mqe_html import (PAGE_HEADER, PAGE_FOOTER,
                      TABLE_HEADER_POSTS, TABLE_ROW_POSTS, TABLE_FOOTER)


def filterfalse_posts(source_folder, reports_folder='../reports-filterfalse/'):
    """Submit for processing all .xml files in source folder."""
    print('Processing forum posts files in {}'.format(source_folder),
          flush=True)
    set_folder(reports_folder)
    results = []
    for forum_name, posts in get_forum_data(source_folder):
        print('Processing {}'.format(forum_name), flush=True)
        results.append(filterfalse_forum(forum_name, posts, reports_folder))
    write_index(source_folder, results, reports_folder)


def filterfalse_forum(forum_name, posts, reports_folder):
    """Write page for forum posts that have links, to results folder."""
    output_file = os.path.join(reports_folder, forum_name + '.html')
    with open(output_file, 'wt', encoding='utf-8', newline='\n') as fout:
        fout.write(PAGE_HEADER.format(forum_name))
        link_count = no_link_count = 0
        current_posts = filter(is_current, posts)
        for post in current_posts:
            if has_link(post):
                link_count += 1
            else:
                postid = post.get('id')
                fout.write('<p><b>Post:</b> {}</p>\n'.format(postid))
                userid = post.find('userid')
                fout.write('<p><b>UserID:</b> {}</p>\n'.format(userid.text))
                created = post.find('created')
                created_date = date.fromtimestamp(int(created.text))
                fout.write('<p><b>Created:</b> {}</p>\n'.format(created_date))
                message_text = post.find('message').text
                fout.write('<p><b>Message:</b></p>\n{}\n<hr>\n'.format(message_text))
                no_link_count += 1
        fout.write(PAGE_FOOTER)
    return forum_name, link_count, no_link_count


def write_index(source_folder, results, reports_folder):
    """Write summary/index file to results folder."""
    with open(reports_folder + 'index.html', 'wt',
              encoding='utf-8', newline='\n') as fout:
        fout.write(PAGE_HEADER.format('Summary for ' + source_folder))
        fout.write(TABLE_HEADER_POSTS)
        for forum_name, posts_with, posts_without in results:
            posts_total = posts_with + posts_without
            posts_with_pct = round((posts_with / posts_total) * 100) if posts_with else 0
            posts_without_pct = 100 - posts_with_pct
            fout.write(TABLE_ROW_POSTS.format(forum_name, forum_name,
                                              posts_with, posts_with_pct,
                                              posts_without, posts_without_pct,
                                              posts_total))
        fout.write(TABLE_FOOTER)
        fout.write(PAGE_FOOTER)


if __name__ == '__main__':
    sys.exit(filterfalse_posts('../forums/'))
