#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Common code used by the framework.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:version: 1.0.0
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

import glob
import os
import pickle
import subprocess
import xml.etree.ElementTree as ET
from collections import Counter
from datetime import date
from typing import Dict, Tuple

from nltk import LancasterStemmer

from mqe_regexs import (HAS_CITATION, HAS_HTTP, HAS_REFERENCES, HAS_YEAR_REF)


def categorise_tokens(tokenised_text: list, taxonomy: Dict[str, str]) -> Dict[str, int]:
    """Return count of matched categorisable terms in source material."""
    stemmed_tokens = [LancasterStemmer().stem(token) for token in tokenised_text]
    stemmed_counts = Counter(stemmed_tokens)
    category_counts = {key: 0 for key in taxonomy.keys()}
    for key, keywords in taxonomy.items():
        for keyword in keywords:
            if keyword in stemmed_counts:
                category_counts[key] += stemmed_counts[keyword]
    category_counts['no_tokens'] = len(stemmed_tokens)
    return category_counts


def extract_text(from_file: str, current_dir: str):
    """Extract text content from a file."""
    head, tail = os.path.split(from_file)
    os.chdir(head)
    subprocess.run([r'C:\Program Files\LibreOffice\program\soffice.exe',
                    '--headless',
                    '--convert-to',
                    'txt:Text', tail],
                   stdin=None,
                   stdout=subprocess.PIPE,
                   stderr=None)
    os.chdir(current_dir)


def get_forum_data(source_folder: str) -> Tuple[str, iter]:
    """Return forum name and parsed XML from each file in source folder."""
    xml_files = glob.iglob(os.path.join(source_folder, '*.xml'))
    for xml_file in xml_files:
        forum_name = os.path.basename(xml_file)[:-4]
        tree = ET.parse(xml_file)
        root = tree.getroot()
        posts = root.iter('post')
        yield (forum_name, posts)


def has_link(post) -> bool:
    """Return True if post has an indicator of external link.

    Cannot use type hinting as I cannot work out how to define an XML Element.

    :param post: single post element extracted from Moodle forum XML
    :type post: Element

    """
    message_text = post.find('message').text
    return True if (HAS_HTTP(message_text) or
                    HAS_REFERENCES(message_text) or
                    HAS_YEAR_REF(message_text) or
                    HAS_CITATION(message_text)) else False


def is_current(post) -> bool:
    """Return True if post is latest version of that student forum post.

    Cannot use type hinting as I cannot work out how to define an XML Element.

    :param post: single post element extracted from Moodle forum XML
    :type post: Element

    """
    return True if post.find('oldversion').text == '0' else False


def load_taxonomy(taxon_set: str, taxonomies_folder: str) -> Dict[str, set]:
    """Return pre-prepared taxonomy.

    Notes: no error handling, the taxonomy and folder just have to exist.

    """
    taxon_file = os.path.join(taxonomies_folder, taxon_set + '.pkl')
    with open(taxon_file, 'rb') as f_txn:
        return pickle.load(f_txn)


def set_folder(folder_name: str):
    """Ensure we have somewhere to write."""
    if not os.path.exists(folder_name):
        os.mkdir(folder_name)


def set_presentation_dates(presentation: str) -> Tuple[date, date]:
    """Return course dates derived from OU month code in a given year.

    Expand elif cascade for other month codes as neeeded.

    :param presentation: two numbers followed by one letter, eg 17K

    """
    start_year = int('20' + presentation[:2])  # cast for date()
    month_code = presentation[2].upper()
    if month_code == 'E':  # E -> forum open Apr-Dec
        start_date = date(start_year, 4, 1)
        end_date = date(start_year, 12, 31)
    elif month_code == 'K':  # K -> forum open Oct-Jun
        start_date = date(start_year, 10, 1)
        end_date = date(start_year + 1, 6, 30)
    return start_date, end_date
