#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Produce stemmed set of keywords to classify material.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:version: 1.0.0
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""


import os
import pickle
import sys
from configparser import ConfigParser

from nltk import LancasterStemmer

from mqe_common import set_folder


def prepare_taxonomy(taxon_set,
                     taxonomies_folder='../taxonomies',
                     results_folder='../taxonomies'):
    """Save stemmed version of taxonomy."""
    print('Preparing {} taxonomy.'.format(taxon_set), flush=True)
    set_folder(results_folder)
    taxonomy = get_taxonomy(taxon_set, taxonomies_folder)
    if taxonomy:
        write_taxonomy(taxon_set, taxonomy, results_folder)
        pickle_taxonomy(taxon_set, taxonomy, results_folder)
        print('Prepared {} taxonomy.'.format(taxon_set), flush=True)
    else:
        print('Preparing {} taxonomy failed'.format(taxon_set), flush=True)
        raise SystemExit('Check taxonomies folder and taxonomy ini file.')


def get_taxonomy(taxon_set, taxonomies_folder):
    """Return stemmed version of taxonomy."""
    cfg = ConfigParser(allow_no_value=True, strict=False)
    cfg.read(os.path.join(taxonomies_folder, taxon_set + '.ini'))
    taxonomy = {}
    for key in cfg.sections():
        taxonomy[key] = set(LancasterStemmer().stem(value[0]) for value
                            in cfg[key].items())
    return taxonomy


def write_taxonomy(taxon_set, taxonomy, results_folder):
    """Write taxonomy to taxon_set txt file in results_folder."""
    with open(os.path.join(results_folder, taxon_set + '_stemmed.txt'),
              'wt', encoding='utf-8', newline='\n') as fout:
        fout.write('Stemmed words prepared for {} taxonomy\n'.format(taxon_set))
        for key in taxonomy:
            fout.write('\n{}:\n'.format(key))
            for value in taxonomy[key]:
                fout.write('{}\n'.format(value))


def pickle_taxonomy(taxon_set, taxonomy, results_folder):
    """Write taxonomy to taxon_set pickle file in results_folder."""
    with open(os.path.join(results_folder, taxon_set + '.pkl'),
              'wb') as fout:
        pickle.dump(taxonomy, fout, pickle.HIGHEST_PROTOCOL)


if __name__ == '__main__':
    sys.exit(prepare_taxonomy('bloom'))
