#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Trace Moodle user IDs across VLE forums.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

import csv
import glob
import os
import sys
from operator import itemgetter

from mqe_common import set_folder

from mqe_html import (PAGE_HEADER, PAGE_FOOTER,
                      TABLE_HEADER_2COLS, TABLE_ROW_2COLS, TABLE_FOOTER)

from mqe_regexs import GET_MODULE_CODE, GET_PRESENTATION_CODE


def get_paths_and_codes(source_folder):
    """Populate dict of userids and list of courses that they have studied."""
    userid_paths = {}
    course_codes = set()
    posts_files = glob.iglob(os.path.join(source_folder, '*_filtered_posts.csv'))
    print('Processing forum posts files in {}'.format(source_folder),
          flush=True)
    for posts_file in posts_files:
        print('Processing {}'.format(posts_file), flush=True)
        module_code = GET_MODULE_CODE(posts_file).group()
        presentation_code = GET_PRESENTATION_CODE(posts_file).group()
        course_code = module_code + '-' + presentation_code
        course_codes.add(course_code)
        update_userid_paths(get_userids(posts_file),
                            userid_paths,
                            course_code)
    return userid_paths, course_codes


def trace_userids(source_folder,
                  data_folder='../data/',
                  reports_folder='../reports/'):
    """Submit for processing all *_posts.csv files in source folder."""
    set_folder(data_folder)
    set_folder(reports_folder)
    userid_paths, course_codes = get_paths_and_codes(source_folder)
    write_userid_paths(userid_paths, reports_folder)
    for_sorting = [course_code.split('-') for course_code in course_codes]
    for_sorting.sort(key=itemgetter(0))  # by course code
    for_sorting.sort(key=itemgetter(1))  # by presentation
    eligible_courses = ['-'.join(course_entry) for course_entry in for_sorting]
    write_userid_course_matrix(userid_paths, eligible_courses, data_folder)


def get_userids(posts_file):
    """Return set of userids that have posted to a forum."""
    with open(posts_file, 'rt', encoding='utf-8', newline='\n') as csv_posts:
        posts_reader = csv.DictReader(csv_posts)
        userids = []
        for post in posts_reader:
            userids.append('{:0>6}'.format(post['UserID']))
    return set(userids)


def update_userid_paths(userids, userid_paths, course_code):
    """Populate dict of userids and list of courses that they have accessed."""
    for userid in userids:
        userid_paths.setdefault(userid, {course_code}).add(course_code)


def write_userid_paths(userid_paths, reports_folder):
    """Write userids and their courses as a table in a web page."""
    output_file = os.path.join(reports_folder, 'study-paths.html')
    with open(output_file, 'wt', encoding='utf-8', newline='\n') as fout:
        fout.write(PAGE_HEADER.format('UserID Study Pathways'))
        fout.write(TABLE_HEADER_2COLS.format('UserID', 'Courses'))
        for userid in sorted(userid_paths):
            courses_taken = ', '.join(sorted(userid_paths[userid]))
            fout.write(TABLE_ROW_2COLS.format(userid, courses_taken))
        fout.write(TABLE_FOOTER)
        fout.write(PAGE_FOOTER)


def get_matrix_row(userid_paths, eligible_courses):
    """Yield one line at a time the matrix of userids and courses studied."""
    for userid in sorted(userid_paths):
        matrix_row = [userid]
        courses_taken = [course if course in userid_paths[userid] else 0
                         for course in eligible_courses]
        matrix_row.extend(courses_taken)
        yield matrix_row


def write_userid_course_matrix(userid_paths, eligible_courses, data_folder):
    """Write matrix of userids and courses studied to csv."""
    output_file = os.path.join(data_folder, 'study-paths.csv')
    with open(output_file, 'wt', encoding='utf-8', newline='\n') as csv_output:
        matrix_writer = csv.writer(csv_output)
        matrix_header = ['UserID']
        matrix_header.extend(eligible_courses)
        matrix_writer.writerow(matrix_header)
        matrix_writer.writerows(get_matrix_row(userid_paths, eligible_courses))


if __name__ == '__main__':
    sys.exit(trace_userids('../data/'))
    # sys.exit(trace_userids('../data-test/',
                  # sys.exit(data_folder='../data-test/',
                  # sys.exit(reports_folder='../reports-test/'))
