#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Track classification of posts during a presentation.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:version: 1.0.0
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

import csv
import glob
import os
import pickle
import sys
from collections import Counter, defaultdict
from datetime import timedelta
from operator import itemgetter

from mqe_common import load_taxonomy, set_folder, set_presentation_dates

from mqe_regexs import GET_PRESENTATION_CODE


def get_classified_forum_posts(source_folder):
    """Return forum name and data from each file in source folder."""
    print('Processing posts files in {}'.format(source_folder),
          flush=True)
    forum_files = glob.iglob(os.path.join(source_folder, '*.pkl'))
    for forum_file in forum_files:
        forum_name = os.path.basename(forum_file)[:-4]
        with open(forum_file, 'rb') as fin:
            yield (forum_name, pickle.load(fin))


def track_classified_forum_posts(source_folder,
                                 taxonomies_folder='../taxonomies',
                                 data_folder='../data/',
                                 taxon_set='bloom'):
    """Submit for processing all pickled files in source folder."""
    set_folder(data_folder)
    taxon_keys = load_taxonomy(taxon_set, taxonomies_folder).keys
    for forum_name, forum_posts in get_classified_forum_posts(source_folder):
        print('Processing {}'.format(forum_name), flush=True)
        forum_posts.sort(key=itemgetter('created'))
        posts_file = os.path.join(data_folder,
                                  forum_name + '_classified_posts.csv')
        write_posts(forum_posts, posts_file,
                    taxon_set, taxon_keys)
        daily_file = os.path.join(data_folder,
                                  forum_name + '_daily_counts.csv')
        classified_file = os.path.join(data_folder,
                                       forum_name + '_classified_counts.csv')
        write_counts(forum_posts, daily_file, classified_file,
                     taxon_set, taxon_keys)


def write_posts(forum_posts, posts_file, taxon_set, taxon_keys):
    """Write metadata for selected posts to csv."""
    with open(posts_file, 'wt', encoding='utf-8', newline='\n') as csv_posts:
        posts_writer = csv.writer(csv_posts)
        header_row = ['PostID',
                      'UserID',
                      'Date'] + [key.capitalize()
                                 for key in taxon_keys()]
        posts_writer.writerow(header_row)
        for post in forum_posts:
            body_row = [post['postid'],
                        post['userid'],
                        post['created']] + [post[taxon_set][key]
                                            for key in taxon_keys()]
            posts_writer.writerow(body_row)


def get_daily_posts(forum_posts):
    """Return list of number of posts for each day."""
    createds = [post['created'] for post in forum_posts]
    count_createds = Counter(createds)
    return sorted([(created, count) for created, count
                   in count_createds.items()])


def write_counts(forum_posts, daily_file, classified_file,
                 taxon_set, taxon_keys):
    """Write daily counts of selected posts to csv."""
    keys = sorted([key for key in taxon_keys()])  # sort to ensure consistency
    with open(daily_file, 'wt',
              encoding='utf-8', newline='\n') as csv_daily, \
        open(classified_file, 'wt',
             encoding='utf-8', newline='\n') as csv_counts:
        daily_writer = csv.writer(csv_daily)
        daily_writer.writerow(['Date',
                               'Count'])
        counts_writer = csv.writer(csv_counts)
        counts_writer.writerow(['Date'] +
                               ['{} Posts'.format(key.capitalize())
                                for key in keys] +
                               ['{} Use'.format(key.capitalize())
                                for key in keys])
        daily_posts = get_daily_posts(forum_posts)
        if not daily_posts:  # forum has no posts
            return
        daily_counts = defaultdict(dict)
        for created, count in daily_posts:
            daily_counts[created]['all_posts'] = count
            for key in keys:
                daily_counts[created][key] = [0, 0]  # list [0] Posts [1] Use
        for post in forum_posts:
            post_date = post['created']
            for key in keys:
                if post[taxon_set][key]:
                    daily_counts[post_date][key][0] += 1
                    daily_counts[post_date][key][1] += post[taxon_set][key]
        next_day = timedelta(days=1)
        presentation = GET_PRESENTATION_CODE(daily_file).group()
        current_date, end_date = set_presentation_dates(presentation)
        first_post = daily_posts[0]
        last_post = daily_posts[-1]
        if daily_counts:
            if first_post[0] < current_date:  # first post before forum opens
                current_date = first_post[0]
            if last_post[0] > end_date:  # last post after course ends
                end_date = last_post[0]
        for post_date in sorted(daily_counts.keys()):
            while post_date != current_date:
                daily_writer.writerow([current_date, 0])
                counts_writer.writerow([current_date] + [0] * len(keys) * 2)
                current_date += next_day
            current_date += next_day
            daily_writer.writerow([post_date,
                                   daily_counts[post_date]['all_posts']])
            counts_writer.writerow([post_date] +
                                   [daily_counts[post_date][key][0]
                                    for key in keys] +
                                   [daily_counts[post_date][key][1]
                                    for key in keys])
        while current_date <= end_date:
            daily_writer.writerow([current_date, 0])
            counts_writer.writerow([current_date] + [0] * len(keys) * 2)
            current_date += next_day


if __name__ == '__main__':
    # sys.exit(track_classified_forum_posts('../posts-classified2/',
                                          # data_folder='../data2/',
                                          # taxon_set='bloom'))
    sys.exit(track_classified_forum_posts('../forum-posts-classified/'))
