#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Track posts that contain external references during a presentation.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:version: 1.0.0
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

import csv
import os
import sys
from collections import Counter
from datetime import date, timedelta
from operator import itemgetter

from mqe_common import (get_forum_data, has_link, is_current,
                        set_folder, set_presentation_dates)

from mqe_regexs import GET_PRESENTATION_CODE


def track_filtered_posts(source_folder, data_folder='../data/'):
    """Process all .xml files in source folder."""
    set_folder(data_folder)
    print('Processing forum posts files in {}'.format(source_folder),
          flush=True)
    for forum_name, posts in get_forum_data(source_folder):
        print('Processing {}'.format(forum_name), flush=True)
        write_metadata(get_posts_metadata(posts), forum_name, data_folder)


def get_posts_metadata(posts):
    """Return metadata for posts in forum."""
    posts_metadata = []
    for post in posts:
        post_metadata = get_post_metadata(post)
        if post_metadata:
            posts_metadata.append(post_metadata)
    posts_metadata.sort(key=itemgetter(2))  # sort on created
    return posts_metadata


def get_post_metadata(post):
    """Return metadata if post is selected."""
    if is_current(post) and has_link(post):
        postid = post.get('id')
        userid = post.find('userid').text
        created = date.fromtimestamp(int(post.find('created').text))
        return (postid, userid, created)
    return None


def write_metadata(posts_metadata, forum_name, data_folder):
    """Write forum posts' metadata to data folder."""
    posts_file = os.path.join(data_folder, forum_name + '_filtered_posts.csv')
    write_posts(posts_metadata, posts_file)
    counts_file = os.path.join(data_folder, forum_name + '_filtered_counts.csv')
    write_counts(posts_metadata, counts_file)


def write_posts(posts_metadata, posts_file):
    """Write metadata for selected posts to csv."""
    with open(posts_file, 'wt', encoding='utf-8', newline='\n') as csv_posts:
        posts_writer = csv.writer(csv_posts)
        posts_writer.writerow(['PostID', 'UserID', 'Date'])
        posts_writer.writerows(posts_metadata)


def get_counts(posts_metadata):
    """Return list of number of posts each day."""
    createds = [created for _, _, created in posts_metadata]
    count_createds = Counter(createds)
    return sorted([(created, count) for created, count in count_createds.items()])


def write_counts(posts_metadata, counts_file):
    """Write daily counts of posts to csv."""
    with open(counts_file, 'wt', encoding='utf-8', newline='\n') as csv_counts:
        counts_writer = csv.writer(csv_counts)
        counts_writer.writerow(['Date', 'Count'])
        counts = get_counts(posts_metadata) if posts_metadata else []
        next_day = timedelta(days=1)
        presentation = GET_PRESENTATION_CODE(counts_file).group()
        current_date, end_date = set_presentation_dates(presentation)
        if counts:
            if counts[0][0] < current_date:  # first post before course starts
                current_date = counts[0][0]
            if counts[-1][0] > end_date:  # last post after course ends
                end_date = counts[-1][0]
        for created, count in counts:
            while created != current_date:
                counts_writer.writerow([current_date, 0])
                current_date += next_day
            current_date += next_day
            counts_writer.writerow([created, count])
        while current_date <= end_date:
            counts_writer.writerow([current_date, 0])
            current_date += next_day


if __name__ == '__main__':
    sys.exit(track_filtered_posts('../forums/'))
    # sys.exit(track_filtered_posts('../forums-test/',
                                  # data_folder='../data-test'))
