#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Identify unique userids for each days' posts in forums.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016-2018 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

import csv
import glob
import os
import sys
from collections import defaultdict, namedtuple
from operator import attrgetter

from mqe_common import set_folder

from mqe_html import (PAGE_HEADER, PAGE_FOOTER,
                      TABLE_HEADER_UNIQUES, TABLE_ROW_UNIQUES, TABLE_FOOTER)

DailyData = namedtuple('Daily_data', 'created, posts, users, userids')


def unique_userids(source_folder,
                   data_folder='../data/',
                   reports_folder='../reports/'):
    """Submit for processing all *_posts.csv files in source folder."""
    set_folder(data_folder)
    set_folder(reports_folder)
    for forum_name, forum_usage in get_forum_usage(source_folder):
        write_usage_report(forum_name, forum_usage, reports_folder)
        write_usage_table(forum_name, forum_usage, data_folder)


def get_forum_usage(source_folder):
    """Return forum name and usage from each file in source folder."""
    print('Processing forum posts files in {}'.format(source_folder),
          flush=True)
    csv_files = glob.iglob(os.path.join(source_folder, '*_classified_posts.csv'))
    for csv_file in csv_files:
        forum_name = os.path.basename(csv_file)[:-21]
        print('Processing {}'.format(forum_name), flush=True)
        forum_usage = prepare_forum_usage(get_users_each_day(csv_file))
        yield (forum_name, forum_usage)


def prepare_forum_usage(userids_by_created):
    """Return comprehensive daily usage data for a forum."""
    forum_usage = []
    for created, userids in userids_by_created.items():
        posts = len(userids)
        userids = tuple(sorted(set(userids)))
        users = len(userids)
        forum_usage.append(DailyData(created=created,
                                     posts=posts,
                                     users=users,
                                     userids=userids))
    forum_usage.sort(key=attrgetter('created'))
    return forum_usage


def get_users_each_day(posts_file):
    """Return daily lists of userids that have posted to a forum."""
    with open(posts_file, 'rt', encoding='utf-8', newline='\n') as csv_posts:
        posts_reader = csv.DictReader(csv_posts)
        userids_by_created = defaultdict(list)
        for post in posts_reader:
            formatted_userid = '{:0>6}'.format(post['UserID'])
            userids_by_created[post['Date']].append(formatted_userid)
    return userids_by_created


def write_usage_report(forum_name, forum_usage, reports_folder):
    """Write usage as a table in a web page."""
    output_file = os.path.join(reports_folder, forum_name + '_usage.html')
    with open(output_file, 'wt', encoding='utf-8', newline='\n') as fout:
        fout.write(PAGE_HEADER.format('Daily usage'))
        fout.write(TABLE_HEADER_UNIQUES)
        for daily_data in forum_usage:
            fout.write(TABLE_ROW_UNIQUES.format(daily_data.created,
                                                daily_data.posts,
                                                daily_data.users,
                                                ', '.join(daily_data.userids)))
        fout.write(TABLE_FOOTER)
        fout.write(PAGE_FOOTER)


def write_usage_table(forum_name, forum_usage, data_folder):
    """Write usage as a table to csv."""
    output_file = os.path.join(data_folder, forum_name + '_usage.csv')
    with open(output_file, 'wt', encoding='utf-8', newline='\n') as csv_output:
        csv_writer = csv.writer(csv_output)
        csv_writer.writerow(['Date', 'Posts', 'Users', 'UserIDs'])
        for daily_data in forum_usage:
            csv_writer.writerow([daily_data.created,
                                 daily_data.posts,
                                 daily_data.users,
                                 ' '.join(daily_data.userids)])


if __name__ == '__main__':
    sys.exit(unique_userids('../data/'))
    # sys.exit(unique_userids('../data-test/',
                            # data_folder='../data-test/',
                            # reports_folder='../reports-test/'))
